#include "globals.h"
#include "bitmaps/clipboard.xbm"


struct FileManager*      fmgr = (struct FileManager *)NULL;


struct FileManager* getFileMgrByFtpHost( host )
  char *host;
{
  int i, l = numFM(fmgr);
  for( i = 0; i < l; i++ )
  {
    struct FileManager *ptr = getIthFM( fmgr, i );
    if( !strcmp( host, ptr->host ) )  return ptr;
  }
  return (struct FileManager *)NULL;
}

int getDirIndexByDirName( f, dragName )
  struct FileManager *f;
  char *dragName;
{
  int i, l = numDir(f->dirInfo);
  for( i = 0; i < l; i++ )
  {
    struct DirInfo *ptr = getIthDir( f->dirInfo, i );
    if( !strcmp( dragName, ptr->di.dragName ) )  return i;
  }
  return -1;
}

void deleteFileManager( w )
  Widget w;
{
  int i, l = numFM(fmgr);

  for( i = 0; i < l; i++ )
  {
    struct FileManager *ptr = getIthFM( fmgr, i );
    if( w == ptr->mainForm )
    {
      XtFree( ptr->cwd );
      if( i == 0 )
      {
	XtCloseDisplay( XtDisplay(w) );
	removeAllFM( &fmgr );
	remove( createPausemeFile() );
	exit( 0 );
      }
      else
      {
	XtDestroyWidget( XtParent(ptr->mainForm) );
	removeIthFM( &fmgr, i );
      }
    }
  }
}

Boolean isAddAccWidget( w )
  Widget w;
{
  struct FileManager *f = fmgr;
  while( f )
  {
    if( w == f->addAccelButton )  return True;
    f = f->next;
  }
  return False;
}

Boolean isaDeviceWidget( w )
  Widget w;
{
  Widget parent = XtParent( w );
  int i, numChildren, nFM = numFM(fmgr), ct;
  WidgetList wl;

  for( ct = 0; ct < nFM; ct++ )
  {
    struct FileManager *f = getIthFM( fmgr, ct );
    if( f->type == FTP_FILESYSTEM )  continue;
    XtVaGetValues( f->devSWRC, XmNchildren, &wl,
                XmNnumChildren, &numChildren, NULL );
    for( i = 0; i < numChildren; i++ )  
      if( parent == wl[i] ) return True;
  }
  return False;
}

void allocateFileManager( parent, type, hostname, username, password )
  Widget parent;
  int type;
  char *hostname, *username, *password;
{
  struct FileManager *f;
  Dimension ht;
  int max, sliderSize;
  XmString xms;
  XWindowAttributes xwa;
  static Atom wmDel;
  extern Pixmap iconPixmap;
  Widget toolbar;

  setWatchCursor();

  f = appendFM( &fmgr );
  f->topSWRC = (Widget)NULL;
  f->type = type;
  f->dirInfo = (struct DirInfo *)NULL;
  f->cmdId = 0;
  f->cmdSocketId = 0;
  f->ftpDataConn = (struct FtpDataConn *)NULL;
  f->cwd = NULL;
  f->cwdPtr = NULL;

  strcpy( f->host, hostname );
  strcpy( f->user, username );
  strcpy( f->pass, password );

  if( numFM(fmgr) == 1 )
  {
    f->parent = XtVaCreatePopupShell( "mfm",
	topLevelShellWidgetClass, parent,
	XmNheight, INIT_HEIGHT, XmNwidth, SL_WIDTH*3+OFFSET, NULL );
    wmDel = XmInternAtom( XtDisplay(f->parent), "WM_DELETE_WINDOW", False );
  }
  else
  {
    if( type == LOCAL_FILESYSTEM )
      f->parent = XtVaCreatePopupShell( "mfm auxillary",
        topLevelShellWidgetClass, getIthFM(fmgr,0)->mainForm,
        XmNheight, INIT_HEIGHT, XmNwidth, SL_WIDTH*3+OFFSET, NULL );
    else
    {
      char name[MAXHOSTNAMELEN+8];
      sprintf( name, "mfm ftp %s", hostname );
      f->parent = XtVaCreatePopupShell( name,
	topLevelShellWidgetClass, getIthFM(fmgr,0)->mainForm,
        XmNheight, INIT_FTP_HEIGHT, XmNwidth, SL_WIDTH*3+OFFSET, NULL );
    }
  }

  XtVaSetValues( f->parent, 
	XmNiconPixmap, iconPixmap, 
	XmNdeleteResponse, XmDO_NOTHING,
	NULL );

  f->mainForm = XtVaCreateManagedWidget( "mainForm",
                xmFormWidgetClass, f->parent,
                NULL );
  XmAddWMProtocolCallback( f->parent, wmDel, quitCB, f );

  /*  create menubar  */

  f->menubar = XmCreateMenuBar( f->mainForm, "menubar", NULL, 0 );
  XtVaSetValues( f->menubar, 
		XmNtopAttachment, XmATTACH_FORM,
		XmNtopOffset, 0,
		XmNleftAttachment, XmATTACH_FORM,
		XmNleftOffset, 0,
		XmNrightAttachment, XmATTACH_FORM,
		XmNrightOffset, 0,
		NULL );

  createMenubar( f, type );
  XtManageChild( f->menubar );

  if( type == LOCAL_FILESYSTEM )
  {
    toolbar = XtVaCreateManagedWidget( "toolbar",
		xmRowColumnWidgetClass, f->mainForm,
		XmNorientation, XmHORIZONTAL,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, f->menubar,
		XmNtopOffset, 5,
		XmNleftAttachment, XmATTACH_FORM,
		XmNleftOffset, 5,
		XmNrightAttachment, XmATTACH_FORM,
		XmNrightOffset, 5,
		NULL );

    createToolbar( f, toolbar );

    f->panedWindow = XtVaCreateWidget( "panedWindow",
                xmPanedWindowWidgetClass, f->mainForm,
                XmNtopAttachment, XmATTACH_WIDGET,
                XmNtopWidget, toolbar,
                XmNtopOffset, 5,
                XmNleftAttachment, XmATTACH_FORM,
                XmNleftOffset, 5,
                XmNrightAttachment, XmATTACH_FORM,
                XmNrightOffset, 5,
                XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 5,
                NULL );
  }
  else
  {
    f->panedWindow = XtVaCreateWidget( "panedWindow",
                xmPanedWindowWidgetClass, f->mainForm,
                XmNtopAttachment, XmATTACH_WIDGET,
                XmNtopWidget, f->menubar,
                XmNtopOffset, 5,
                XmNleftAttachment, XmATTACH_FORM,
                XmNleftOffset, 5,
                XmNrightAttachment, XmATTACH_FORM,
                XmNrightOffset, 5,
                XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 5,
                NULL );
  }

  if( type == LOCAL_FILESYSTEM )
  {
    Pixmap clipboardPixmap;
    Pixel fg, bg;

    f->topForm = XtVaCreateManagedWidget( "topForm",
		xmFormWidgetClass, f->panedWindow,
		XmNpaneMinimum, 130,
                NULL );
		
    f->topSW = XtVaCreateManagedWidget( "topSW",
                xmScrolledWindowWidgetClass, f->topForm,
                XmNtopAttachment, XmATTACH_FORM,
                XmNtopOffset, 0,
                XmNleftAttachment, XmATTACH_FORM,
                XmNleftOffset, 0,
                XmNrightAttachment, XmATTACH_FORM,
                XmNrightOffset, 75,
                XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 5,
                XmNshadowThickness, 2,
                XmNscrollVertical, True,
                XmNscrollHorizontal, True,
                XmNscrollingPolicy, XmAUTOMATIC,
                NULL );

    f->topSWRC = XtVaCreateManagedWidget( "topSWRC",
                xmRowColumnWidgetClass, f->topSW,
		XmNpacking, XmPACK_COLUMN,
                XmNnumColumns, 10,
                XmNorientation, XmVERTICAL,
		XmNuserData, (XtPointer)f,
                NULL );
    XtVaSetValues( f->topSW, XmNworkWindow, f->topSWRC, NULL );

    XtVaGetValues( f->topForm, XmNbackground, &bg, XmNforeground, &fg, NULL );
    clipboardPixmap = XCreatePixmapFromBitmapData( XtDisplay(f->topForm),
        DefaultRootWindow(XtDisplay(f->topForm)),
        (char *)clipboard_bits, clipboard_width, clipboard_height,
        fg, bg, DefaultDepthOfScreen(XtScreen(f->topForm)) );

    f->addAccelButton = XtVaCreateManagedWidget( "addAccelButton",
                xmDrawnButtonWidgetClass, f->topForm,
                XmNtopAttachment, XmATTACH_FORM,
                XmNtopOffset, 0,
                XmNrightAttachment, XmATTACH_FORM,
                XmNrightOffset, 0,
                XmNleftAttachment, XmATTACH_OPPOSITE_FORM,
                XmNleftOffset, -70,
                XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 5,
                XmNlabelType, XmPIXMAP,
                XmNlabelPixmap, clipboardPixmap,
		XmNuserData, (XtPointer)f,
                NULL );
    registerDropSite( f->addAccelButton, ACC_DROP_SITE );
  }

  f->bottomForm = XtVaCreateManagedWidget( "bottomForm",
                xmFormWidgetClass, f->panedWindow,
                XmNpaneMinimum, 150,
                NULL );

  if( type == LOCAL_FILESYSTEM )
  {
    Widget devLabel;
    XFontStruct *font;
    XmFontList  fontList;

    f->devForm = XtVaCreateManagedWidget( "devForm",
                xmFormWidgetClass, f->panedWindow,
                XmNpaneMinimum, 130,
                XmNpaneMaximum, 130,
                NULL );

    f->devSW = XtVaCreateManagedWidget( "devSW",
                xmScrolledWindowWidgetClass, f->devForm,
                XmNtopAttachment, XmATTACH_FORM,
                XmNtopOffset, 5,
                XmNleftAttachment, XmATTACH_FORM,
                XmNleftOffset, 35,
                XmNrightAttachment, XmATTACH_FORM,
                XmNrightOffset, 0,
                XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 5,
                XmNshadowThickness, 2,
                XmNscrollVertical, True,
                XmNscrollHorizontal, True,
                XmNscrollingPolicy, XmAUTOMATIC,
                NULL );

    f->devSWRC = XtVaCreateManagedWidget( "devSWRC",
                xmRowColumnWidgetClass, f->devSW,
                XmNpacking, XmPACK_COLUMN,
                XmNnumColumns, 10,
                XmNorientation, XmVERTICAL,
                XmNuserData, (XtPointer)f,
                NULL );
    XtVaSetValues( f->devSW, XmNworkWindow, f->devSWRC, NULL );

    font = XLoadQueryFont( XtDisplay(parent), "*misc*fixed*bold*14*" );
    fontList = XmFontListCreate( font, "set1" );
    xms = XmStringCreateLtoR( "D\nE\nV\nI\nC\nE\nS", "set1" );
    devLabel = XtVaCreateManagedWidget( "devLabel",
		xmLabelWidgetClass, f->devForm,
		XmNtopAttachment, XmATTACH_FORM,
                XmNtopOffset, 5,
                XmNleftAttachment, XmATTACH_FORM,
                XmNleftOffset, 5,
                XmNbottomAttachment, XmATTACH_FORM,
                XmNbottomOffset, 5,
		XmNfontList, fontList,
		XmNlabelString, xms,
		NULL );
    XmStringFree( xms );
    XmFontListFree( fontList );
  }

  XtManageChild( f->panedWindow );

  if( type == LOCAL_FILESYSTEM )
  {
    if( numFM(fmgr) == 1 )
    {
      Pixel fg, bg;
      XtVaGetValues( f->addAccelButton,
                   XmNforeground, &fg, XmNbackground, &bg, NULL );
      parseApplicationResources( f->addAccelButton, fg, bg );
      parseDevApplicationResources( f->devSWRC, fg, bg );
    }
    else
    {
      Pixel fg, bg;
      XtVaGetValues( f->addAccelButton,
                   XmNforeground, &fg, XmNbackground, &bg, NULL );
      createDeviceIcons( f->devSWRC, fg, bg );
    }
  }

  createFileSelectionBox( f, type );
  XtManageChild( f->parent );

  if( type == LOCAL_FILESYSTEM )
  {
    if( numFM(fmgr) == 1 )
      readRCFile( f->topSWRC );
    else
      loadAccels( f->topSWRC );
  }

  XFlush( XtDisplay(f->parent) );
  XSync( XtDisplay(f->parent), False );

  XtVaGetValues( f->clipWindow, XmNheight, &ht, NULL );
  f->ht = ht - 2;
  XtVaSetValues( f->form, XmNheight, f->ht, NULL );

  XtVaGetValues( f->sb,
        XmNmaximum, &max,
        XmNsliderSize, &sliderSize,
        NULL );
  XmScrollBarSetValues( f->sb,
        max-sliderSize,
        sliderSize, SL_WIDTH, SL_WIDTH, True );

  unsetWatchCursor();
}

void updateAllFileManagers( )
{
  int i, l = numFM(fmgr);
  for( i = 0; i < l; i++ )
    updateFileManager( getIthFM( fmgr, i ) );
}
